#%%

import math
import random

from flight.case import *
from flight.color import Color
from flight.client import *
from flight.expressions import *
from flight.expressions import Expression
from flight.server import FlightServer

from constants import *


#%%

client.activate()

def SplitTriangle(width, height, bottom=0, color=TriangleColor, **kwargs):
    with Group(**kwargs):
        for side, multiplier in [('left', -1), ('right', 1)]:
            with Path(tags=side, color=color * (0.8 - 0.1 * multiplier)):
                MoveTo(multiplier * width / 2, height / 2)
                LineTo(0, -height / 2)
                LineTo(0, height / 2 - bottom)
                FillPath()
        if bottom:
            with Path(tags='bottom', color=color * 0.8):
                MoveTo(-width / 2, height / 2)
                LineTo(0, height / 2 - bottom)
                LineTo(width / 2, height / 2)
                FillPath()


def Star(radius):
    MoveTo(radius / 2, 0)
    for k in range(1, 10):
        theta = 2 * math.pi * k / 10
        r = (1 + (k % 2)) * radius / 2
        LineTo(r * math.cos(theta), r * math.sin(theta))
    ClosePath()


with Screen('main', width=1920, height=1080) as screen:
    with Pattern('beads', width=BeadSeparation * 2, height=BeadSeparation * 2) as beads:
        for x in range(3):
            for y in range(3):
                odd = (x + y) % 2 == 1
                with Group(x=x * beads.width / 2, y=(y + (x % 2) / 3) * beads.height / 2):
                    with Path(tags={'bead', 'odd' if odd else 'even'}, color=OddBeadColor if odd else EvenBeadColor):
                        Ellipse(0, 0, BeadRadius, BeadRadius)
                        FillPath()
    with Pattern('mask_pattern', width=screen.width, height=screen.height) as container:
        with Group('master_mask', color=MaskColor, x=container.width/2, y=container.height/2):
            with Path(tags='background'):
                Ellipse(x=0, y=0, radiusX=MaskOuterRadius, radiusY=MaskOuterRadius)
                FillPath()
            with Path(tags='background patterning', pattern='beads'):
                Ellipse(x=0, y=0, radiusX=MaskInnerRadius, radiusY=MaskInnerRadius)
                FillPath()
            with Group('triangles'):
                for j in range(TriangleCount):
                    with Group(rotate=j / TriangleCount):
                        SplitTriangle(y=-TriangleRadius, tags='triangle', width=TriangleWidth, height=TriangleHeight)
            with Group('mouth', y=MouthOffset):
                with Path(tags='lips background'):
                    RoundedRect(-MouthWidth / 2 - LipThickness, -MouthHeight / 2 - LipThickness,
                                MouthWidth + LipThickness * 2, MouthHeight + LipThickness * 2, MouthCornerRadius + LipThickness)
                    FillPath()
                with Path(tags='hole', color=HoleColor):
                    RoundedRect(-MouthWidth / 2, -MouthHeight / 2, MouthWidth, MouthHeight, MouthCornerRadius)
                    FillPath()
                with Path(tags='tongue inside', color=DarkMaskColor, y=MouthHeight / 2, rotate=0.5):
                    RoundedRect(-MouthWidth / 2, 0, MouthWidth, TongueHeight, MouthCornerRadius)
                    FillPath()
                with Path(tags='liner', color=EyelinerColor, lineWidth=EyelinerThickness, lineDash=EyelinerThickness):
                    RoundedRect(-MouthWidth / 2, -MouthHeight / 2, MouthWidth, MouthHeight, MouthCornerRadius)
                    StrokePath()
            with Group('thirdeye', x=0, y=-ThirdEyeOffset):
                with Path(tags='background'):
                    Ellipse(0, 0, radiusX=ThirdEyeXRadius, radiusY=ThirdEyeYRadius)
                    FillPath()
                with Path(tags='liner', color=EyelinerColor, lineWidth=EyelinerThickness, lineDash=EyelinerThickness):
                    Ellipse(0, 0, radiusX=ThirdEyeXRadius - EyeRimThickness, radiusY=ThirdEyeYRadius - EyeRimThickness)
                    StrokePath()
            with Group('eyes'):
                for side, multiplier in [('left', -1), ('right', 1)]:
                    with Group(tags={'eye', side}, x=multiplier * (EyeSeparation/2 + EyeXRadius)):
                        with Path(tags='inside', color=DarkMaskColor):
                            Ellipse(x=0, y=0, radiusX=EyeXRadius, radiusY=EyeYRadius)
                            FillPath()
                        SplitTriangle(tags='eyelid upper', y=-EyeYRadius / 2, width=TriangleWidth, height=TriangleHeight)
                        SplitTriangle(tags='eyelid lower', y=EyeYRadius / 2, width=TriangleWidth, height=-TriangleHeight)
                        with Path():
                            Ellipse(x=0, y=0, radiusX=EyeXRadius, radiusY=EyeYRadius)
                            with Path(tags='pupil hole', color=HoleColor):
                                Rect(-PupilWidth / 2, -PupilHeight / 2, PupilWidth, PupilHeight)
                                FillPath()
                        with Path(tags='rim background', lineWidth=EyeRimThickness):
                            Ellipse(x=0, y=0, radiusX=EyeXRadius + EyeRimThickness/2, radiusY=EyeYRadius + EyeRimThickness / 2)
                            StrokePath()
                        with Path(tags='liner', color=EyelinerColor, lineWidth=EyelinerThickness, lineDash=EyelinerThickness):
                            Ellipse(x=0, y=0, radiusX=EyeXRadius, radiusY=EyeYRadius)
                            StrokePath()
            SplitTriangle(id='nose', width=NoseWidth, height=NoseHeight, bottom=NoseBottom)
            with Group('helmet', fader=0):
                with Path(color=TriangleColor * 0.5, fader=0.6):
                    Ellipse(x=0, y=0, radiusX=MaskOuterRadius, radiusY=MaskOuterRadius)
                    FillPath()
                with Path(color='white', lineWidth=10):
                    Ellipse(x=0, y=0, radiusX=MaskOuterRadius, radiusY=MaskOuterRadius)
                    StrokePath()
                with Group(color='white'):
                    with Path():
                        Ellipse(x=0, y=0, radiusX=MaskOuterRadius, radiusY=MaskOuterRadius, startAngle=0.15, endAngle=0.35)
                        FillPath()
                    with Path():
                        Ellipse(x=0, y=0, radiusX=MaskOuterRadius, radiusY=MaskOuterRadius, startAngle=0.55, endAngle=0.95)
                        FillPath()
            with Path(id='symbol', x=0, y=-ThirdEyeOffset):
                Ellipse(0, 0, radiusX=ThirdEyeXRadius - EyeRimThickness, radiusY=ThirdEyeYRadius - EyeRimThickness)
                Text(font='120px NotoColorEmoji, NotoEmoji, emoji', textAlign='center', textBaseline='middle', color=HoleColor)
    with Group('stars', composite='lighter', x=screen.width / 2, y=screen.height / 2, fader=0):
        for j in range(100):
            with Group(tags='star', lineWidth=2):
                with Path(tags='fill'):
                    Star(10)
                    FillPath()
                with Path(tags='outline'):
                    Star(10)
                    StrokePath()
    with Path(id='mask', pattern='mask_pattern', fader=0):
        Rect(0, 0, container.width, container.height)
        FillPath()
